#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###
#
# Rapidshare Download is the legal property of Raul Gonzalez Duque <zootropo@gmail.com>
# Copyright (c) 2007 Raul Gonzalez Duque
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###

import cookielib, urllib, urllib2
import time, sys
import subprocess
import re
import os
import ConfigParser
import getpass

try:
    import pygtk
    pygtk.require("2.0")
    import gtk
    with_gtk = True
except ImportError:
    with_gtk = False



def log(text):
    """Prints a message with the local time and date."""
    date = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    print "[%s] - %s" % (date, text)

def from_rapidshare(url):
    """Check if this is a rapidshare link"""
    return (url.startswith("rapidshare.com") or 
            url.startswith("www.rapidshare.com") or 
            url.startswith("http://rapidshare.com") or
            url.startswith("http://www.rapidshare.com") or
            url.startswith("rapidshare.de") or 
            url.startswith("www.rapidshare.de") or 
            url.startswith("http://rapidshare.de") or
            url.startswith("http://www.rapidshare.de"))


# There needs to be at least one argument
if len(sys.argv) < 2:
    print "Too few arguments"
    print "Try --help for more information"
    exit()


# Get the login info from the config file, or ask for it
if os.environ.has_key("APPDATA") and os.path.exists(os.environ["APPDATA"]):
    path = os.environ["APPDATA"] + "/rapidshare-dl.ini"
else:
    path = os.path.expanduser("~") + "/.rapidshare-dl"

if not os.path.exists(path):
    user = raw_input("Enter your user name: ")
    password = getpass.getpass(prompt="Enter your password: ")
    print
else:
    cfg = ConfigParser.SafeConfigParser()
    cfg.readfp(file(path))
    try:
        user = cfg.get("Login", "user")
        password = cfg.get("Login", "password")
    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
        print "The config file is corrupt"
        user = raw_input("Enter your user name: ")
        password = getpass.getpass(prompt="Enter your password: ")
        print
        os.remove(path)


# Check if the login is right
cred = urllib.urlencode({"login": user, "password": password})
req = urllib2.urlopen("https://ssl.rapidshare.com/cgi-bin/premiumzone.cgi", cred)
cookie = req.headers.get("set-cookie", "")

if cookie:
    log("Logged in as %s" % user)
    print
    if not os.path.exists(path):
        config = "[Login]\n"
        config += "user = %s\n" % user
        config += "password = %s\n" % password
        config_file = open(path, "w")
        config_file.write(config)
        config_file.close()
else:
    log("Invalid user name or password")
    exit()


# Process the arguments
if sys.argv[1] == "-h" or sys.argv[1] == "--help":
    print "rapidshare-dl http://rapidshare.com/files/FILE1 ... http://rapidshare.com/files/FILEN"
    print "  Download one or several rapidshare links passed as argument\n"

    print "rapidshare-dl http://some-web-site.com"
    print "  Download a list of links from an URL\n"

    print "rapidshare-dl links.txt"
    print "  Download a list of links from a file\n"

    print "rapidshare-dl -c"
    print "  Download a list of links from the clipboard (PyGTK has to be installed)\n"
    exit()

from_file = False
urls = []
if os.path.exists(sys.argv[1]):    # If this is a file
    from_file = True
    l_file = sys.argv[1]
    log("Reading list of links from the file %s" % l_file)
    print "\n"
    list_f = file(l_file, "r")
    urls = list_f.readlines()
    list_f.close()
elif sys.argv[1].startswith("http://"):    # If this is a URL
    if from_rapidshare(sys.argv[1]):
        for i in range(1, len(sys.argv)):
            urls.append(sys.argv[i])
    else:
        log("This is not a Rapidshare URL. Trying to retrieve links from " + sys.argv[1])
        try:
            source = "\n".join(urllib.urlopen(sys.argv[1]).readlines())
        except IOError:
            print "Operation timed out"
            exit()
        urls_source = re.findall(r'<a[^>]*?href="(?P<href>[^"]*?)".*?>', source)
        for url in urls_source:
            if not url in urls and from_rapidshare(url):
                if not url.startswith("http://"):
                    url = "http://" + url
                url += "\n"
                urls.append(url)
        print
        if urls:
            log("I found %s links. Saving to rapidsharetmp.txt" % len(urls))
            l_file = "rapidsharetmp.txt"
            list_f = file(l_file, "w")
            list_f.write("".join(urls))
            list_f.close()
            from_file = True
            print urls
            print
        else:
            log("I found 0 links.")
            exit()
elif sys.argv[1] == "-c" or sys.argv[1] == "--clipboard":
    if not with_gtk:
        print "PyGTK is not available in your system. This feature cannot be used"
        exit()
    else:
        log("Trying to retrieve links from the clipboard")
        clipboard = gtk.clipboard_get()
        text = clipboard.wait_for_text() + "\n"
        urls_source = re.findall("rapidshare\.com/files/.{8}/.*[\s]", text)
        urls_source += re.findall("rapidshare\.de/files/.{8}/.*[\s]", text)
        for url in urls_source:
            url = "http://" + url + "\n"
            if not url in urls:
                urls.append(url)
        print
        if urls:
            log("I found %s links. Saving to rapidsharetmp.txt" % len(urls))
            l_file = "rapidsharetmp.txt"
            list_f = file(l_file, "w")
            list_f.write("".join(urls))
            list_f.close()
            from_file = True
            print urls
            print
        else:
            log("I found 0 links.")
            exit()
else:
    print "%s is not a valid argument." % sys.argv[1]
    print "Please use a URL or URLs from Rapidshare, a file with a list of Rapidshare URLs, or the address of a site with Rapidshare URLs."
    print "If this is a URL, please ensure that it starts with http://"
    print "If this is a file, please ensure that it exists"
    exit()



# Download the files
current = urls
errors = ""
for url in urls:
    try:
        url = url.strip()

        print url

        process = subprocess.Popen(["wget", "-c", url, "--http-user", user, "--http-password", password])
        process.wait()

        if from_file:
            current = current[1:]
            list_f = file(l_file, "w")
            list_f.writelines([errors] + current)
            list_f.close()
            log("List updated")

        print "\n\n"
    except KeyboardInterrupt:
        print "\n\nBye"
        exit()
    except ValueError:
        log("This doesn't look like an url. We won't download it\n\n")
        if from_file:
            errors = errors + current[0]
            current = current[1:]
    except urllib2.URLError, e:
        log("URLError, we can't download this\n\n")
        print e
        if from_file:
            errors = errors + current[0]
            current = current[1:]
    except:
        log("Unexpected error.")
        raise

if errors:
    log("The following files could not be downloaded")
    print errors
else:
    log("All files were downloaded")

log("Finished")



